# coding: utf-8
"""
レンダリング画像に絞り回折効果のフィルターを適用します
"""

__author__ = "CASPAR003 <caspar003@caspar003.info>"
__date__ = "2014-07-20/2014-07-28"
__version__ = "1.2"

def ca3_pe_aperturediffraction(img, F):
    """
    画像に絞り回折効果のフィルターを適用します
    @param img フィルタをかけたい画像（Shade imageオブジェクト）
    @param F   F値
    
    計算には時間がかかるので巨大な画像に適用するときはお気を付けて．
    私の環境では512×512pxの画像で30秒くらいかかります．
    """
    import math
    import numpy as np
    
    fheight = min(img.size)     # 画像の短辺長さ [px]
    x = 1.22 * 550.0e-9 * F     # エアリーディスク半径 [m]
    xp = x * fheight / 0.024    # エアリーディスク半径 [px]
    
    # x座標ベクトル
    sx = 0.0 if (img.size[0] % 2 == 0) else 0.5
    vecx = np.arange(img.size[0])
    vecx = vecx + (sx - img.size[0]/2.0)
    vecx = vecx / xp
    vecx = vecx**2
    
    # y座標ベクトル
    sy = 0.0 if (img.size[1] % 2 == 0) else 0.5
    vecy = np.arange(img.size[1])
    vecy = vecy + (sy - img.size[1]/2.0)
    vecy = vecy / xp
    vecy = vecy**2
    
    # 回折カーネル
    matc = np.array([vecx + y for y in vecy])
    matc = np.sqrt(matc)
    matc = np.sinc(matc)
    matc = matc**2
    sumc = np.sum(matc)
    matc = matc/sumc
    
    # 作業用配列の確保
    matr = np.zeros(img.size)
    matg = np.zeros(img.size)
    matb = np.zeros(img.size)
    
    # Shade imageをnumpy arrayに代入
    for ix in range(img.size[0]):
        for iy in range(img.size[1]):
            matr[ix, iy], matg[ix, iy], matb[ix, iy] = img.get_pixel(ix, iy)
    
    # 配列を1次元配列に変更
    matr = matr.reshape(img.size[0] * img.size[1])
    matg = matg.reshape(img.size[0] * img.size[1])
    matb = matb.reshape(img.size[0] * img.size[1])
    matc = matc.reshape(img.size[0] * img.size[1])
    
    # 1次元に引き延ばしたときにカーネル中心がベクトル中心からずれてしまうので，ベクトルの最初の方を切り落とす．
    # 切り落とし量を計算
    filterbegin = 0
    if (img.size[0] % 2 == 0): filterbegin += 1
    if (img.size[1] % 2 == 0): filterbegin += img.size[1]
    
    # フィルターを適用
    matr = np.convolve(matr, matc[filterbegin:], "same")    # 部分配列をつかってカーネルを切り出している
    matg = np.convolve(matg, matc[filterbegin:], "same")    # 
    matb = np.convolve(matb, matc[filterbegin:], "same")    # 
    
    # 配列を元の形に戻す
    matr = matr.reshape(img.size)
    matg = matg.reshape(img.size)
    matb = matb.reshape(img.size)
    
    # numpy arrayをShade imageに戻す
    for ix in range(img.size[0]):
        for iy in range(img.size[1]):
            img.set_pixel(ix, iy, (matr[ix, iy], matg[ix, iy], matb[ix, iy]))
    return



if (__name__ == "__main__"):
    img = xshade.scene().rendering.image
    if (img is not None):
        ca3_pe_aperturediffraction(img, 8.0)
        xshade.scene().rendering.update_image()
